﻿/*----------------------------------------------------------------
//  <copyright file="DemoStore.cs" company="EaCloud项目团队">
//      Copyright (c) 2020-2021 SeonSoft. All rights reserved.
//  </copyright>
//  <site>http://www.seonsoft.biz</site>
//  <last-editor>Seon.Hu</last-editor>
//  <last-date>2021-04-17 19:06</last-date>
//----------------------------------------------------------------*/

using EaCloud;
using EaCloud.Data;
using EaCloud.Entity;
using EaCloud.EventBuses;
using EaCloud.Mapping;
using EaCloud.Template.Pack.Demo.Core.Dtos;
using EaCloud.Template.Pack.Demo.Core.Events;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using System;
using System.Linq;
using System.Threading.Tasks;
using DemoEntity = EaCloud.Template.Pack.Demo.Core.Entities.Demo;

namespace EaCloud.Template.Pack.Demo.Core
{
    #region "Demo存储功能"
    /// <summary>
    /// Demo存储功能
    /// </summary>
    public class DemoStore : IDemoStore
    {
        #region "属性"
        #region "获取 服务提供者对象"
        /// <summary>
        /// 获取 服务提供者对象
        /// </summary>
        protected IServiceProvider ServiceProvider { get; }
        #endregion
        #region "获取 事件总线"
        /// <summary>
        /// 获取 事件总线
        /// </summary>
        protected IEventBus EventBus => ServiceProvider.GetService<IEventBus>();
        #endregion
        #region "获取 日志对象"
        /// <summary>
        /// 获取 日志对象
        /// </summary>
        protected ILogger Logger => ServiceProvider.GetLogger(GetType());
        #endregion

        #region "获取 Demo实体仓储模型"
        /// <summary>
        /// 获取 Demo实体仓储模型
        /// </summary>
        protected IRepository<DemoEntity, long> DemoRepository => ServiceProvider.GetService<IRepository<DemoEntity, long>>();
        #endregion

        #region "获取 Demo实体查询数据集"
        /// <summary>
        /// 获取 Demo实体查询数据集
        /// </summary>
        public virtual IQueryable<DemoEntity> Demos => DemoRepository.QueryAsNoTracking();
        #endregion

        #endregion

        #region "构造函数"
        #region "初始化一个Demo存储功能的新实例"
        /// <summary>
        /// 初始化一个Demo存储功能（<see cref="DemoStore"/>）的新实例
        /// </summary>
        /// <param name="provider">服务提供者</param>
        public DemoStore(IServiceProvider provider)
        {
            ServiceProvider = provider;
        }
        #endregion

        #endregion

        #region "方法"
        #region "新增"
        /// <summary>
        /// 新增
        /// </summary>
        /// <param name="demoEntities">数据实体</param>
        /// <returns></returns>
        public OperationResult Insert(params DemoEntity[] demoEntities)
        {
            Check.NotNull(demoEntities, nameof(demoEntities));

            var operationResult = DemoRepository.Insert(demoEntities);
            if (operationResult.Succeeded)
            {
                //触发事件
                DemoEventData eventData = new() { OperateType = OperateType.Insert, Demos = demoEntities };
                EventBus?.Publish(eventData);
            }
            return operationResult;
        }
        #endregion
        #region "异步新增"
        /// <summary>
        /// 异步新增
        /// </summary>
        /// <param name="demoEntities">数据实体</param>
        /// <returns></returns>
        public async Task<OperationResult> InsertAsync(params DemoEntity[] demoEntities)
        {
            Check.NotNull(demoEntities, nameof(demoEntities));

            var operationResult = await DemoRepository.InsertAsync(demoEntities);
            if (operationResult.Succeeded)
            {
                //触发事件
                DemoEventData eventData = new() { OperateType = OperateType.Insert, Demos = demoEntities };
                await EventBus?.PublishAsync(eventData);
            }
            return operationResult;
        }
        #endregion
        #region "更新"
        /// <summary>
        /// 更新
        /// </summary>
        /// <param name="demoEntities">数据实体</param>
        /// <returns></returns>
        public OperationResult Update(params DemoEntity[] demoEntities)
        {
            Check.NotNull(demoEntities, nameof(demoEntities));

            var operationResult = DemoRepository.Update(demoEntities);
            if (operationResult.Succeeded)
            {
                //触发事件
                DemoEventData eventData = new() { OperateType = OperateType.Update, Demos = demoEntities };
                EventBus?.Publish(eventData);
            }
            return operationResult;
        }
        #endregion
        #region "异步更新"
        /// <summary>
        /// 异步更新
        /// </summary>
        /// <param name="demoEntities">数据实体</param>
        /// <returns></returns>
        public async Task<OperationResult> UpdateAsync(params DemoEntity[] demoEntities)
        {
            Check.NotNull(demoEntities, nameof(demoEntities));

            DemoEventData eventData = new() { OperateType = OperateType.Update };
            var operationResult = OperationResult.NoChanged;
            foreach (var demoEntity in demoEntities)
            {
                Check.GreaterThan(demoEntity.Id, nameof(demoEntity.Id), 0);

                var entity = DemoRepository.Get(demoEntity.Id);
                if (entity == null)
                {
                    return new OperationResult(OperationResultType.ValidError, $"编号为“{demoEntity.Id}”的 {nameof(demoEntity)} 不存在。");
                }
                operationResult = await DemoRepository.UpdateAsync(demoEntity);
                if (!operationResult.Succeeded)
                {
                    return operationResult;
                }
                eventData.Demos.Add(demoEntity);
            }
            if (eventData.HasData())
            {
                await EventBus?.PublishAsync(eventData);
            }
            return operationResult;
        }
        #endregion
        #region "删除"
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="demoEntities">数据实体</param>
        /// <returns></returns>
        public OperationResult Delete(params DemoEntity[] demoEntities)
        {
            Check.NotNull(demoEntities, nameof(demoEntities));

            var operationResult = DemoRepository.Delete(demoEntities);
            if (operationResult.Succeeded)
            {
                //触发事件
                DemoEventData eventData = new() { OperateType = OperateType.Delete, Demos = demoEntities };
                EventBus?.Publish(eventData);
            }
            return operationResult;
        }
        #endregion
        #region "异步删除"
        /// <summary>
        /// 异步删除
        /// </summary>
        /// <param name="keys">数据实体编号</param>
        /// <returns></returns>
        public async Task<OperationResult> DeleteAsync(long[] keys)
        {
            Check.NotNull(keys, nameof(keys));

            DemoEventData eventData = new() { OperateType = OperateType.Delete };
            var operationResult = OperationResult.NoChanged;
            foreach (var key in keys)
            {
                Check.GreaterThan(key, nameof(key), 0);

                var entity = DemoRepository.Get(key);
                if (entity == null)
                {
                    return new OperationResult(OperationResultType.ValidError, $"编号为“{key}”的 Demo 不存在。");
                }
                operationResult = await DemoRepository.DeleteAsync(entity);
                if (!operationResult.Succeeded)
                {
                    return operationResult;
                }
                eventData.Demos.Add(entity);
            }
            if (eventData.HasData())
            {
                await EventBus?.PublishAsync(eventData);
            }
            return operationResult;
        }
        #endregion

        #region "异步以DTO为载体新增"
        /// <summary>
        /// 异步以DTO为载体新增
        /// </summary>
        /// <param name="dtos">DTO信息</param>
        /// <returns>业务操作结果</returns>
        public virtual async Task<OperationResult> InsertByDtoAsync(params DemoInputDto[] dtos)
        {
            Check.NotNull(dtos, nameof(dtos));
            Check.Validate<DemoInputDto, long>(dtos, nameof(dtos));

            DemoEventData eventData = new() { OperateType = OperateType.Insert };
            OperationResult result = await DemoRepository.InsertAsync(dtos, null,
              async (dto, entity) =>
               {
                   eventData.Demos.Add(entity);
                   return await Task.FromResult(entity);
               });
            if (result.Succeeded)
            {
                if (eventData.HasData())
                {
                    await EventBus?.PublishAsync(eventData);
                }
            }
            return result;
        }
        #endregion
        #region "异步以DTO为载体更新"
        /// <summary>
        /// 异步以DTO为载体新增
        /// </summary>
        /// <param name="dtos">DTO信息</param>
        /// <returns>业务操作结果</returns>
        public virtual async Task<OperationResult> UpdateByDtoAsync(params DemoInputDto[] dtos)
        {
            Check.NotNull(dtos, nameof(dtos));
            Check.Validate<DemoInputDto, long>(dtos, nameof(dtos));

            DemoEventData eventData = new() { OperateType = OperateType.Update };
            OperationResult result = await DemoRepository.UpdateAsync(dtos,
             async (dto, entity) =>
               {
                   Check.GreaterThan(dto.Id, nameof(dto.Id), 0);

                   var dbEntity = DemoRepository.Get(dto.Id);
                   if (dbEntity == null)
                   {
                       result = new OperationResult(OperationResultType.ValidError, $"编号为“{dto.Id}”的 Demo 不存在。");
                   }
                   eventData.Demos.Add(entity);
                   await Task.FromResult(0);
               });
            if (result.Succeeded)
            {
                if (eventData.HasData())
                {
                    await EventBus?.PublishAsync(eventData);
                }
            }
            return result;
        }
        #endregion
        #region "异步以DTO为载体更新"
        /// <summary>
        /// 异步以DTO为载体新增
        /// </summary>
        /// <param name="dtos">DTO信息</param>
        /// <returns>业务操作结果</returns>
        public virtual async Task<OperationResult> UpdateByDtoAsync(params DemoDto[] dtos)
        {
            Check.NotNull(dtos, nameof(dtos));

            DemoEventData eventData = new() { OperateType = OperateType.Update };
            var operationResult = OperationResult.NoChanged;
            foreach (var dto in dtos)
            {
                Check.GreaterThan(dto.Id, nameof(dto.Id), 0);

                var dbEntity = DemoRepository.Get(dto.Id);
                if (dbEntity == null)
                {
                    return new OperationResult(OperationResultType.ValidError, $"编号为“{dto.Id}”的 Demo 不存在。");
                }

                var demoEntity = dto.MapTo<DemoEntity>();
                operationResult = await DemoRepository.UpdateAsync(demoEntity);
                if (!operationResult.Succeeded)
                {
                    return operationResult;
                }
                eventData.Demos.Add(demoEntity);
            }

            if (operationResult.Succeeded)
            {
                if (eventData.HasData())
                {
                    await EventBus?.PublishAsync(eventData);
                }
            }
            return operationResult;
        }
        #endregion

        #endregion

    }
    #endregion

}
